// File: main.cpp

#include <iostream>
#include <string>
#include <fstream>
using namespace std;

// Set up some aliases for readability.
using number_of_records = int;
using connection_status = bool;

const int MAX_STUDENTS = 100;

// Define Student struct.
struct Student {
  int sid;
  string first;
  string last;
};


// Function declarations, a.k.a. function prototypes.
connection_status isFileConnectedForInput(const string& filename, ifstream& ifs);
connection_status isFileConnectedForOutput(const string& filename, ofstream& ofs);

number_of_records readStudentsFromFile(ifstream& ifs, Student students[]);
number_of_records readStudentsFromFileCSV(ifstream& ifs, Student students[]);
void writeStudentsToFile(ofstream& ofs, const Student students[], const number_of_records count);
void displayStudents(const Student students[], const number_of_records count);



int main() {
  Student students[MAX_STUDENTS];
  
  ifstream ifs;
  ofstream ofs;

  if ( !isFileConnectedForInput("studentsCSV.txt", ifs) ) {
    cerr << "Error connecting to file." << endl;
    exit(1);
  }

  number_of_records count = readStudentsFromFileCSV(ifs, students);
  displayStudents(students, count);

  if ( !isFileConnectedForOutput("studentsOut.txt", ofs) ) {
    cerr << "Error connecting to file." << endl;
    exit(1);
  }

  writeStudentsToFile(ofs, students, count);

  return 0;
}// end main()




// Function definitions
connection_status isFileConnectedForInput(const string& filename, ifstream& ifs) {
  ifs.open(filename);
  return ifs.is_open();
}// end isFileConnectedForInput()

connection_status isFileConnectedForOutput(const string& filename, ofstream& ofs) {
  ofs.open(filename);
  return ofs.is_open();
}// end isFileConnectedForOutput()

number_of_records readStudentsFromFileCSV(ifstream& ifs, Student students[]) {
  // Use local vars to hold each field.
  string sidText;
  string first;
  string last;
  number_of_records count = 0;

  getline(ifs, sidText, ',');
  while ( !ifs.eof() ) {
    getline(ifs, first, ',');
    getline(ifs, last);

    // Create the Student instance and store in the array.
    students[count++] = {stoi(sidText), first, last};

    getline(ifs, sidText, ',');
  }
  ifs.close();
  return count;
}// end readStudentsFromFileCSV()

number_of_records readStudentsFromFile(ifstream& ifs, Student students[]) {
  /* Note: This function reads a space delimited file using the >> operator.
   * It is not appropriate however when the file is a CSV file. Use the getline
   * for those files.
   */
  // Use local vars to hold each field.
  int sid;
  string first;
  string last;
  number_of_records count = 0;

  ifs >> sid;
  while ( !ifs.eof() ) {
    ifs >> first;
    ifs >> last;

    // Create the Student instance and store in the array.
    students[count++] = {sid, first, last};

    ifs >> sid;
  }
  ifs.close();
  return count;
}// end readStudentsFromFile()

void writeStudentsToFile(ofstream& ofs, const Student students[], const number_of_records count) {
  for (int i = 0; i < count; i++) {
    ofs << "[" << students[i].sid << "]"
        << "[" << students[i].first << " " << students[i].last << "]"
        << endl;
  }
  ofs.close();
}

void displayStudents(const Student students[], const number_of_records count) {
  for (int i = 0; i < count; i++) {
    cout << "[" << students[i].sid << "]"
         << "[" << students[i].first << " " << students[i].last << "]"
         << endl;
  }
}