// File: DateTime.h

// Note: Class interface file.

// Guards to prevent multiple inclusions of the header file.
#ifndef DATETIME_H
#define DATETIME_H

#include <string>
#include <iostream>
using namespace std;

class DateTime {
private:
  // Later C++ standards allow for field initialization (preferred).
  int year = 2000;
  int month = 1;
  int day = 1;
  // Time will be stored in 24 hour format.
  int hour = 0;
  int minute = 0;
  int second = 0;

public:  
  // Ctors
  // Version A: Include multiple overlaads.
  // DateTime();
  // DateTime(const int year, const int month, const int day,
  //          const int hour, const int minute, const int second,
  //          const bool ampm);

  // Version B: Use optional parameters. These default value should only
  // appear in the prototype, not the method header.
  DateTime(const int year = 2000, const int month = 1, const int day = 1,
           const int hour = 0, const int minute = 0, const int second = 0);

  // Getters/setters
  // const method: makes it an accessor. This means
  // the method cannot update the object in an way (direct or inderict).
  int getYear() const;
  int getMonth() const;
  int getDay() const;
  int getHour() const;
  int getMinute() const;
  int getSecond() const;

  // Setters
  void setYear(const int year);
  void setMonth(const int month);
  void setDay(const int day);
  void setHour(const int hour);
  void setMinute(const int minute);
  void setSecond(const int second);

  void setDate(const int year, const int month, const int day);
  void setTime(const int hour, const int minute, const int second);
  void setDateTime(const int year, const int month, const int day,
                   const int hour, const int minute, const int second);

  // Descriptor
  string toString() const;


  // Behavioral methods
  bool isLeap() const;
};

// Overloaded operator <<(). This must be a function, not a method.
// Left hand object is always a ostream type.≠
ostream& operator <<(ostream& os, const DateTime& o);

#endif
