// File: main.cpp

#include <iostream>
#include <string>
#include <fstream>
using namespace std;

// Set up some aliases for enhanced readability.
using number_of_students = int;
using connection_status = bool;

const int MAX_STUDENTS = 100;

// Define the Student struct.
struct Student {
  int sid;
  string first;
  string last;
};


// Function declarations, a.k.a. function prototypes.
connection_status isInputFileConnected(const string& filename, ifstream& ifs);
connection_status isOutputFileConnected(const string& filename, ofstream& ofs);

number_of_students readStudentsFromFile(ifstream& ifs, Student students[]);
number_of_students readStudentsFromFileCSV(ifstream& ifs, Student students[]);

void writeStudentsToFile(ofstream& ofs, const Student students[], const number_of_students count);
void displayStudents(const Student students[], const number_of_students count);



int main() {
  Student students[MAX_STUDENTS];
  ifstream ifs;
  ofstream ofs;

  if ( !isInputFileConnected("studentsCSV.txt", ifs) ) {
    cerr << "Unable to connect to file." << endl;
    exit(1);
  }

  number_of_students count = readStudentsFromFileCSV(ifs, students);

  displayStudents(students, count);

  if ( !isOutputFileConnected("studentsOut.txt", ofs) ) {
    cerr << "Unable to connect to file." << endl;
    exit(1);
  }

  writeStudentsToFile(ofs, students, count);
  return 0;
}// end main()


// Function definitions.
connection_status isInputFileConnected(const string& filename, ifstream& ifs) {
  ifs.open(filename);
  return ifs.is_open();
}

connection_status isOutputFileConnected(const string& filename, ofstream& ofs) {
  ofs.open(filename);
  return ofs.is_open();
}

number_of_students readStudentsFromFile(ifstream& ifs, Student students[]) {
  int sid;
  string first;
  string last;
  number_of_students count = 0;

  ifs >> sid;
  while ( !ifs.eof() ) {
    ifs >> first >> last;

    students[count++] = {sid, first, last};
    // cout << sid << " " << first << " " << last << endl;

    ifs >> sid;
  }
  ifs.close();
  return count;
}

number_of_students readStudentsFromFileCSV(ifstream& ifs, Student students[]) {
  string sidText;
  string first;
  string last;
  number_of_students count = 0;

  getline(ifs, sidText, ',');
  while ( !ifs.eof() ) {
    getline(ifs, first, ',');
    getline(ifs, last);

    students[count++] = { stoi(sidText), first, last};
    // cout << sid << " " << first << " " << last << endl;

    getline(ifs, sidText, ',');
  }
  ifs.close();
  return count;
}


void writeStudentsToFile(ofstream& ofs, const Student students[], const number_of_students count) {
  for (int i = 0; i < count; i++) {
    ofs << "[" << students[i].sid << "]"
         << "[" << students[i].first << " " << students[i].last << "]"
         << endl;
  }
}


void displayStudents(const Student students[], const number_of_students count) {
  for (int i = 0; i < count; i++) {
    cout << "[" << students[i].sid << "]"
         << "[" << students[i].first << " " << students[i].last << "]"
         << endl;
  }
}
