// File: DateTime.cpp

#include "DateTime.h"
#include <string>
#include <iostream>
#include <sstream>

using namespace std;


// DateTime::DateTime() {
//   // Option A: call setters

//   // Option B: assign new object to this.
//   *this = {2000, 1, 1, 0, 0, 0, true};
// }// end

DateTime::DateTime(const int year, const int month, const int day,
                   const int hour, const int minute, const int second) {
  setDateTime(year, month, day, hour, minute, second);
}// end

int DateTime::getYear() const   { return year;   }// end
int DateTime::getMonth() const  { return month;  }// end
int DateTime::getDay() const    { return day;    }// end
int DateTime::getHour() const   { return hour;   }// end
int DateTime::getMinute() const { return minute; }// end
int DateTime::getSecond() const { return second; }// end

// Setters
void DateTime::setYear(const int year) {
  // If the new value is invalid, don't change the object.
  this->year = (year > 0)? year : this->year;
}// end

void DateTime::setMonth(const int month) {
  // If the new value is invalid, don't change the object.
  this->month = (month >= 1 && month <= 12)? month : this->month;
}// end

void DateTime::setDay(const int day) {
  // If the new value is invalid, don't change the object.
  int daysInMonth[] = {0,
                       31, 28, 31,
                       30, 31, 30,
                       31, 31, 30,
                       31, 30, 31
                       }; 
  daysInMonth[2] = isLeap()? 29 : 28;
  // this->day = (day >= 1 && day <= daysInMonth[month])? day : this->day;
  if (day < 1 || day > daysInMonth[month]) {
    throw range_error("Invalid day: " + to_string(day));
  }

}// end

void DateTime::setHour(const int hour) {
  // hour: 0-23
  this->hour = (hour < 0 || hour > 23)? 0 : hour;
}// end

void DateTime::setMinute(const int minute) {
  // minute: 0-59
  this->minute = (minute < 0 || minute > 59)? 0 : minute;
}// end

void DateTime::setSecond(const int second) {
  // second: 0-59
  this->second = (second < 0 || second > 59)? 0 : second;
}// end

void DateTime::setDate(const int year, const int month, const int day) {
  setYear(year);
  setMonth(month);
  setDay(day);
}// end

void DateTime::setTime(const int hour, const int minute, const int second) {
  setHour(hour);
  setMinute(minute);
  setSecond(second);
}// end

void DateTime::setDateTime(const int year, const int month, const int day,
                           const int hour, const int minute, const int second) {
  setDate(year, month, day);
  setTime(hour, minute, second);
}// end

// Descriptor
string DateTime::toString() const { 
  ostringstream oss;

  oss << year 
      << "/" 
      << month 
      << "/" 
      << day
      << " "
      << hour 
      << ":" 
      << minute 
      << ":"  
      << second;

  return oss.str();
}// end


bool DateTime::isLeap() const {
  bool isCentury = (year % 100 == 0);
  bool is_div_4 = (year % 4 == 0);
  bool is_div_400 = (year % 400 == 0);

  return (!isCentury && is_div_4) || is_div_400;
}// end


ostream& operator <<(ostream& os, const DateTime& o) {
  os << o.toString();
  return os;
}// end