// File: Temp.cpp

#include "Temp.h"
#include <sstream>
#include <iomanip>

using namespace std;


double Temp::inDegreesCelcius() const {
    if ( scale == "F") {
        return (value - 32.0) / 1.8;
    }
    return value;
}// end convertToCelcius()

double Temp::inDegreesFahrenheit() const {
    if ( scale == "C") {
        return value * 1.8 + 32.0;
    }
    return value;
}// end convertToFahrenheit()

Temp::Temp(const double value, const string& scale) {
    setScale(scale);
    setValue(value);
}// end Temp()

Temp::~Temp() {}// end ~Temp()

double Temp::getValue() const { return value; }// end getValue
string Temp::getScale() const { return scale; }// end getScale()

void Temp::setValue(const double value)  { this->value = value; }// end setValue()
void Temp::setScale(const string& scale) { this->scale = scale; }// end setScale()

string Temp::toString() const {
    // Format using current scale.
    return toString(scale);
}// end toString()

string Temp::toString(const string& formatScale) const {
    const string SYMBOL_DEGREES = "\u00B0";

    ostringstream oss;
    double outputValue = value;
    string outputScale = formatScale;

    if ( scale != formatScale ) {
        if      ( formatScale == "F") { outputValue = inDegreesFahrenheit(); } 
        else if ( formatScale == "C") { outputValue = inDegreesCelcius();    }
    }

    oss << setiosflags(ios::fixed) << setprecision(1)
        << outputValue << SYMBOL_DEGREES << " " << outputScale;

    return oss.str();
}// end toString()

// Overloaded method operators (+)
Temp Temp::operator +(const double valueIncrement) { return Temp { value + valueIncrement, scale }; }// end +()
Temp Temp::operator -(const double valueIncrement) { return Temp { value - valueIncrement, scale }; }// end -()

Temp Temp::operator +(const Temp& addendTemp) {
    // The return Temp object will have this's scale.
    string returnedScaled = scale;
    double returnedValue = value + addendTemp.value;

    if ( addendTemp.scale != scale) {
        if      ( scale == "F" ) { returnedValue = value + addendTemp.inDegreesFahrenheit(); }
        else if ( scale == "C" ) { returnedValue = value + addendTemp.inDegreesCelcius();    }
    }

    return Temp { returnedValue, returnedScaled };
}// end +()


bool Temp::operator ==(const Temp& rhs) { return scale == rhs.scale && value == rhs.value;          }// end ==()
bool Temp::operator !=(const Temp& rhs) { return !(*this == rhs);                                   }// end !=()
bool Temp::operator <(const Temp& rhs)  { return inDegreesFahrenheit() < rhs.inDegreesFahrenheit(); }// end <()
bool Temp::operator <=(const Temp& rhs) { return *this < rhs || *this == rhs;                       }// end <=()
bool Temp::operator >(const Temp& rhs)  { return !(*this <= rhs);                                   }// end >()
bool Temp::operator >=(const Temp& rhs) { return !(*this < rhs);                                    }// end >=()

/* Overloaded function operators (<<)
 * This must be a function overload, since the stream object
 * is required on the left-hand-side (lfs) of the operator.
 */

ostream& operator <<(ostream& os, const Temp& o) {
    /* Call the .toString() method to do all the heavy lifting. */
    os << o.toString();
    return os;
}