// File: main.cpp

#include "Temp.h"
#include <iostream>
#include <fstream>
#include <string>
#include <iomanip>

using namespace std;

//typedef int number_of_records_t;  // C++03 standard
using number_of_records_t = int;    // C++11 standard - preferred

// Set up the array size constant
const int MAX_TEMP_COUNT = 100;

// Function declarations. Streams are always passed by reference.
bool openFileForInput(ifstream& ifs, const string& filename);
bool openFileForOutput(ofstream& ofs, const string& filename);

void writeTempsToFile(ofstream& ofs, const Temp temps[], const int numberOfRecords);
number_of_records_t loadTempsFromFile(ifstream& ifs, Temp temps[]);

number_of_records_t processInputFile(const string& filename, Temp temps[]);
void processOutputFile(const string& filename, const Temp temps[], const int numberOfRecords);
/*--------------------------------------------------------------------------------------------------*/

int main() {
    Temp temps[MAX_TEMP_COUNT];
    int numberOfRecords = processInputFile("temps.txt", temps);

    processOutputFile("tempsOut.txt", temps, numberOfRecords);

    // Use overloaded << to output to the screen as well.
    cout << "List of temps:"
         << endl;

    for (int i = 0; i <= numberOfRecords - 1; i++) {
        cout << setw(10) << "Original: " 
             << setw(9) << temps[i] 
             << setw(14) << "Converted: " 
             << setw(9) << ( (temps[i].getScale() == "F")? temps[i].toString("C") : temps[i].toString("F") )
             << endl;
    }

    cout << endl;

    return 0;
}// end main()
/*--------------------------------------------------------------------------------------------------*/

number_of_records_t processInputFile(const string& filename, Temp temps[]) {
    ifstream ifs;

    if ( !openFileForInput(ifs, filename) ) {
        cerr << "Unable to read file." << endl;
        exit(1);
    }

    int numberOfRecords = loadTempsFromFile(ifs, temps);

    ifs.close();
    return numberOfRecords;
}// end processInputFile()
/*--------------------------------------------------------------------------------------------------*/

void processOutputFile(const string& filename, const Temp temps[], const int numberOfRecords) {
    ofstream ofs;

    if ( !openFileForOutput(ofs, filename) ) {
        cerr << "Unable to create file." << endl;
        exit(1);
    }

    writeTempsToFile(ofs, temps, numberOfRecords);

    ofs.close();
}// end processOutputFile()
/*--------------------------------------------------------------------------------------------------*/

bool openFileForInput(ifstream& ifs, const string& filename) {
    ifs.open(filename);
    return ifs.is_open();
}// end openFileForInput()
/*--------------------------------------------------------------------------------------------------*/

bool openFileForOutput(ofstream& ofs, const string& filename) {
    ofs.open(filename);
    return ofs.is_open();
}// end openFileForOutput()
/*--------------------------------------------------------------------------------------------------*/

void writeTempsToFile(ofstream& ofs, const Temp temps[], const int numberOfRecords) {
    for (int count = 1; count <= numberOfRecords; count++) {
        ofs << temps[count - 1] << endl;
    }
}// end writeTempsToFile()

number_of_records_t loadTempsFromFile(ifstream& ifs, Temp temps[]) {
    int numberOfRecords = 0;
    string valueString;
    string scale;

    // Read from the file, a string value into valueString, using the comma (,)
    // as the delimiter.
    getline(ifs, valueString, ',');
    while ( !ifs.eof() ) {
        // Read into scale, using \n as the delimiter
        getline(ifs, scale);

        // Record was read. Now store in the array as a Temp object.
        temps[numberOfRecords++] = Temp { stod(valueString), scale };

        // Read first first of subsequent record
        getline(ifs, valueString, ',');
    }

    return numberOfRecords;
}// end loadTempsFromFile()