/* File: ArrayList.h */

#ifndef ARRAYLIST_H
#define ARRAYLIST_H

#include <string>           // to_string
#include <iostream>
#include <sstream>          // ostringstream
#include <stdexcept>        // range_error

using namespace std;

const int MAX_ELEMENTS = 100;

template <typename E>
class ArrayList {
private:
    E elements[MAX_ELEMENTS];
    int numberOfElements = 0;

public:
    /* The initial value is the default value for the type (E). */
    ArrayList() { 
        for (int i = 0; i <= MAX_ELEMENTS - 1; i++) {
            elements[i] = E{};
        }
    }

    virtual ~ArrayList() {}
    virtual  void pushBack(const E& element) final  { pushAt(numberOfElements, element); }
    virtual  void pushFront(const E& element) final { pushAt(0, element);                }

    virtual void pushAt(const int index, const E& element) final {
        if (numberOfElements == MAX_ELEMENTS || index < 0 || index > numberOfElements) {
            throw range_error( "Invalid index: " + to_string(index) );
        }

        /* Push only if space is available and index is valid */
        for (int i = numberOfElements - 1; i >= index; i--) {
            elements[i + 1] = elements[i];
        }
        elements[index] = element;
        numberOfElements++;
    }

    virtual  E popBack() final  { return popAt(numberOfElements - 1); }
    virtual  E popFront() final { return popAt(0);                    }

    virtual  E popAt(const int index) final {
        /* Pop only if index is valid. */
        if (index < 0 || index >= numberOfElements) {
            throw range_error( "Invalid index: " + to_string(index) );
        }
        
        /* Save the value being removed. */
        E valuePopped = elements[index];

        /* Shift all elements after index to the left by one. */
        for (int i = index + 1; i <= numberOfElements - 1; i++) {
            elements[i - 1] = elements[i];
        }
        numberOfElements--;
        return valuePopped;
    }

    virtual E getBack() const final   { return getAt(numberOfElements - 1); }
    virtual  E getFront() const final { return getAt(0);                    }

    virtual E getAt(const int index) const final  {
        /* Return the element at index, or throw an exception if index is invalid. */
        if (index < 0 || index >= numberOfElements) { throw range_error( "Invalid index: " + to_string(index) ); }
        
        return elements[index];
    }

    virtual void setAt(const int index, const E& element) final {
        /* Update the element at index, or throw an exception if index is invalid. */
        if (index < 0 || index >= numberOfElements) { throw range_error( "Invalid index: " + to_string(index) ); }
        
        elements[index] = element;
    }

    virtual int size() const { return numberOfElements; }

    virtual string toString() const  {
        ostringstream oss;

        oss << "size: " << numberOfElements << ", [ ";
        for (int i = 0; i <= numberOfElements - 2; i++) {
            oss << elements[i] << ", ";
        }
        oss << elements[numberOfElements - 1] << " ]";

        return oss.str();
    }
};


template <typename E>
ostream& operator <<(ostream& os, const ArrayList<E>& o) { 
    os << o.toString();
    return os;
}

#endif