// File: ArrayList.h

#ifndef ARRAY_LIST_H
#define ARRAY_LIST_H

#include <string>
#include <iostream>
#include <sstream>
#include <stdexcept>

using namespace std;

template <typename E>
class ArrayList {
private:
    using EPtr = E *;

    EPtr elements = nullptr;

    // These are private fields that are exposed as public
    // properties to the client code. See public: below
    int f_count = 0;
    bool f_isEmpty = true;

    // Utility methods
    void copyElements(const ArrayList& copy) {
        f_count = copy.count;
        f_isEmpty = copy.isEmpty;
        elements = new E[count];

        for (int i = 0; i <= count - 1; i++) {
            elements[i] = copy.elements[i];
        }
    }// end copyElements()
   
    void releaseElements() {
        delete [] elements;
        elements = nullptr;
        f_count = 0;
        f_isEmpty = true;
    }// releaseElements()

public:
    // count and isEmpty are now a readonly public properties.
    const int& count = f_count;
    const bool& isEmpty = f_isEmpty;

    // Default constructor: dctor
    ArrayList() {}// end ArrayList()

    // Copy constructor: cctor
    ArrayList(const ArrayList& copy) { 
        copyElements(copy); 
    }// end ArrayLit()

    // Desctructor: dtor
    virtual ~ArrayList() { 
        releaseElements();
    }// end ~ArrayList()

    // Copy assignment
    const ArrayList& operator =(const ArrayList& rhs) {
        if ( this != &rhs ) {
            releaseElements();
            copyElements(rhs);
        }
        return *this;
    }// end =()

    void insertFront(const E& element) {
        insertAt(0, element);
    }// end insertFront()

    void insertBack(const E& element) {
        insertAt(count, element);
    }// end insertBack()

    void insertAt(const int index, const E& element) {
        if ( index < 0 || index > count ) {
            throw range_error("Invalid index: " + to_string(index) );
        }

        if ( isEmpty ) {
            // Very first insertion into an empty array.
            elements = new E[1] { element };
            f_isEmpty = false;

        } else {
            // First increase array size by 1.
            EPtr tmpElements = new E[count + 1];

            tmpElements[index] = element;

            // Copy all the elements.
            for (int fromIndex = 0, toIndex = 0; fromIndex <= count - 1; fromIndex++) {
                if (fromIndex == index) {
                    toIndex += 1;
                }
                tmpElements[toIndex++] = elements[fromIndex];
            }

            // Now release the elements array; not needed anymore
            delete [] elements;

            // Assign 'tmpElements' to 'elements'.
            elements = tmpElements;
            tmpElements = nullptr;
        }

        f_count += 1;
    }// end insertAt()

    E removeFront() {
        return removeAt(0);
    }// end removeFront()

    E removeBack() {
        return removeAt(count - 1);
    }// end removeBack()
    
    E removeAt(const int index) {
        // Remove only if index is in range.
        if (index < 0 || index >= count) {
            throw range_error( "Invalid index: " + to_string(index) );
        }

        // Cache element removed.
        E elementRemoved = elements[index];

        /* If we are removing the last element no copying is needed. */
        if ( count == 1 ) {
            releaseElements();
            return elementRemoved;

        } else {
            // Decrease the array size by 1.
            EPtr tmpElements = new E[count - 1];

            for (int fromIndex = 0, toIndex = 0; fromIndex <= count - 1; fromIndex++) {
                if (fromIndex == index) {
                    continue;
                }
                tmpElements[toIndex++] = elements[fromIndex];
            }

            // Release the elements array; not needed anymore.
            delete [] elements;

            // Reassign elements.
            elements = tmpElements;
            tmpElements = nullptr;

            f_count -= 1;
            return elementRemoved;
        }
    }// end removeAt()

    virtual string toString() const {
        ostringstream oss;

        if ( isEmpty ) {
            oss << "count: " << count << ", [ ]";

        } else {
            oss << "count: " << count << ", [ ";

            for (int i = 0; i <= count - 1; i++) {
                oss << elements[i] << ", ";
            }

            // Remove the last ', ' from the stream.
            oss.seekp(-2, ios::end);
            oss << " ]";
        }

        return oss.str();
    }// end
};// end ArrayList

template <typename E>
ostream& operator <<(ostream& os, const ArrayList<E>& o) {
    os << o.toString();
    return os;
}// end <<()

#endif