# Class: LinkedList

```cpp
#ifndef LINKEDLIST_H
#define LINKEDLIST_H

#include <string>
#include <sstream>
#include <iostream>
#include <stdexcept>

using namespace std;


/* This class will be a bi-directional linked list, using a head and tail dummy node. 
 * E represents the element's type
 *     head                                                   tail
 *      |                                                      |
 * (nullptr, X, next)<-->(previous, E, next)<-->...(previous, X, nullptr)
 */
template <typename E>
class LinkedList {
private:
    class Node;
    using NodePtr =  Node *;
    /*---------------------------------------------------------------------*/

    class Node {
    public:
        NodePtr previous;
        E element;
        NodePtr next;

        Node(NodePtr const previous, const E& element, NodePtr const next)
            : previous(previous), element(element), next(next)
        {}// end Node()
    };
    /*---------------------------------------------------------------------*/
    /* Utility methods. */

    NodePtr createNode(NodePtr const previous, 
                       const E element, NodePtr 
                       const next);
    void insertNodeAfter(NodePtr const before, const E& element);
    E removeNode(NodePtr& del);
    void releaseNode(NodePtr& del)
    void copyNodes(const LinkedList& copy);
    void releaseNodes();
    /*---------------------------------------------------------------------*/

    /* Fields. */
    NodePtr head = nullptr;
    NodePtr tail = nullptr;
    int count = 0;
    /*---------------------------------------------------------------------*/

public:
    LinkedList(); 
    LinkedList(const LinkedList& copy);
    ~LinkedList();

    const LinkedList& operator =(const LinkedList& rhs);

    /* Position-based methods */
    E getFront() const;
    E getBack() const;
    E getAt(const int index) const

    void setFront(const E& element);
    void setBack(const E& element);
    void setAt(const int index, const E& element);

    void pushFront(const E element);
    void pushBack(const E element);
    void pushAt(const int index, const E element);
    
    E popFront();
    E popBack();
    E popAt(const int index);
 
    /* Value-based methods */
    void push(const E element);
    E pop(const E& element);


    // /* Misc methods */
    bool isEmpty() const
    int size() const;
    bool contains(const E& element) const;
    
    string toString() const;

}; // end LinkedList
/*---------------------------------------------------------------------*/

template <typename E>
ostream& operator <<(ostream& os, const LinkedList<E>& o);

#endif
```